@@ -1,25 +1,26 @@
 /*----------------------------------------------------------------------------
   ChucK Strongly-timed Audio Programming Language
-    Compiler and Virtual Machine
+    Compiler, Virtual Machine, and Synthesis Engine
 
   Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
+  it under the dual-license terms of EITHER the MIT License OR the GNU
+  General Public License (the latter as published by the Free Software
+  Foundation; either version 2 of the License or, at your option, any
+  later version).
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful and/or
+  interesting, but WITHOUT ANY WARRANTY; without even the implied warranty
+  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+  MIT Licence and/or the GNU General Public License for details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  U.S.A.
+  You should have received a copy of the MIT License and the GNU General
+  Public License (GPL) along with this program; a copy of the GPL can also
+  be obtained by writing to the Free Software Foundation, Inc., 59 Temple
+  Place, Suite 330, Boston, MA 02111-1307 U.S.A.
 -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
@@ -40,7 +41,6 @@
 #include "chuck_errmsg.h"
 
 
-
 //-----------------------------------------------------------------------------
 // name: enum te_Type
 // desc: basic, default ChucK types
@@ -75,12 +75,18 @@ typedef enum { te_globalTypeNone,
 
 //-----------------------------------------------------------------------------
 // name: enum te_HowMuch
-// desc: how much to scan/type check
+// desc: how much to scan/type-check/emit
 //-----------------------------------------------------------------------------
 typedef enum {
-    te_do_all = 0, te_do_classes_only, te_do_no_classes
+    te_do_all = 0,
+    te_do_import_only, // attend only to things to be imported | 1.5.4.0 (ge)
+    te_skip_import, // do everything except things to be imported | 1.5.4.0 (ge)
 } te_HowMuch;
 
+// function to if a function matches a particular criteria
+t_CKBOOL howMuch_criteria_match( te_HowMuch criteria, a_Func_Def func_def );
+t_CKBOOL howMuch_criteria_match( te_HowMuch criteria, a_Class_Def class_def );
+
 
 
 
@@ -169,11 +175,15 @@ public:
     // -1 base, 0 current, 1 climb
     T lookup( S_Symbol xid, t_CKINT climb = 1 )
     {
-        Chuck_VM_Object * val = NULL; assert( scope.size() != 0 );
+        Chuck_VM_Object * val = NULL;
+        assert( scope.size() != 0 );
 
         if( climb == 0 )
         {
-            val = (*scope.back())[xid];
+            // find key
+            std::map<S_Symbol, Chuck_VM_Object *> * b = scope.back();
+            std::map<S_Symbol, Chuck_VM_Object *>::iterator it = b->find(xid);
+            val = it != b->end() ? it->second : NULL;
             // look in commit buffer if the back is the front
             if( !val && scope.back() == scope.front()
                 && (commit_map.find(xid) != commit_map.end()) )
@@ -183,7 +193,10 @@ public:
         {
             for( t_CKUINT i = scope.size(); i > 0; i-- )
             {
-                val = (*scope[i - 1])[xid];
+                // find key
+                std::map<S_Symbol, Chuck_VM_Object *> * b = scope[i-1];
+                std::map<S_Symbol, Chuck_VM_Object *>::iterator it = b->find(xid);
+                val = it != b->end() ? it->second : NULL;
                 if( val ) break;
             }
 
@@ -193,7 +206,10 @@ public:
         }
         else
         {
-            val = (*scope.front())[xid];
+            // look front
+            std::map<S_Symbol, Chuck_VM_Object *> * b = scope.front();
+            std::map<S_Symbol, Chuck_VM_Object *>::iterator it = b->find(xid);
+            val = it != b->end() ? it->second : NULL;
             // look in commit buffer
             if( !val && (commit_map.find(xid) != commit_map.end()) )
                 val = commit_map[xid];
@@ -366,15 +382,13 @@ struct Chuck_Context : public Chuck_VM_Object
     // error - means to free nspc too
     t_CKBOOL has_error;
 
-    // AST parse tree (does not persist past context unloading)
+    // AST (does not persist past context unloading)
     a_Program parse_tree;
-    // AST public class def if any (does not persist past context unloading)
-    a_Class_Def public_class_def;
 
     // progress
-    enum { P_NONE = 0, P_CLASSES_ONLY, P_ALL_DONE };
+    enum ContextProgress { P_NONE = 0, P_IMPORTING, P_IMPORTED, P_ALL_DONE };
     // progress in scan / type check / emit
-    t_CKUINT progress;
+    ContextProgress progress;
 
     // things to release with the context
     std::vector<Chuck_VM_Object *> new_types;
@@ -390,8 +404,7 @@ public:
 public:
     // constructor
     Chuck_Context() { parse_tree = NULL; nspc = new Chuck_Namespace;
-                      public_class_def = NULL; has_error = FALSE;
-                      progress = P_NONE; }
+                      has_error = FALSE; progress = P_NONE; }
     // destructor
     virtual ~Chuck_Context();
 
@@ -646,6 +659,65 @@ public:
 
 
 
+//-----------------------------------------------------------------------------
+// name: struct Chuck_ArrayTypeKey
+// desc: a 4-tuple key | 1.5.4.0 (ge & nick) added
+//-----------------------------------------------------------------------------
+struct Chuck_ArrayTypeKey
+{
+    Chuck_Type * array_parent;
+    t_CKUINT depth;
+    Chuck_Type * base_type;
+
+    // constructor
+    Chuck_ArrayTypeKey( Chuck_Type * p, t_CKUINT d, Chuck_Type * t )
+    : array_parent(p), depth(d), base_type(t) { }
+
+    // comparator
+    bool operator<( const Chuck_ArrayTypeKey & rhs ) const;
+};
+struct Chuck_ArrayTypeKeyCmp
+{
+    // operator overload (for map)
+    bool operator()( const Chuck_ArrayTypeKey & a, const Chuck_ArrayTypeKey & b ) const;
+};
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: struct Chuck_ArrayTypeCache
+// desc: cache for array types | 1.5.4.0 (ge & nick & andrew) added
+//-----------------------------------------------------------------------------
+struct Chuck_ArrayTypeCache
+{
+public:
+    // constructor
+    Chuck_ArrayTypeCache() : m_enabled(FALSE) { }
+    // destructor
+    virtual ~Chuck_ArrayTypeCache() { clear(); }
+    // clear it
+    void clear();
+    // enable or disable
+    void enable( t_CKBOOL yesOrNo ) { m_enabled = yesOrNo; }
+    // get whether enabled
+    t_CKBOOL isEnabled() const { return m_enabled; }
+    // lookup an array type; if not already cached, create and insert
+    Chuck_Type * getOrCreate( Chuck_Env * env,
+                              Chuck_Type * array_parent,
+                              t_CKUINT depth,
+                              Chuck_Type * base_type );
+
+protected:
+    // the cache
+    std::map<Chuck_ArrayTypeKey, Chuck_Type *, Chuck_ArrayTypeKeyCmp> cache;
+    // is the cache currently enabled?
+    t_CKBOOL m_enabled;
+};
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: struct Chuck_Env
 // desc: chuck type environment; one per VM instance
@@ -674,7 +746,7 @@ public:
     t_CKBOOL is_global();
     // global namespace
     Chuck_Namespace * global();
-    // user namespace, if there is one (if not, return global)
+    // user namespace
     Chuck_Namespace * user();
     // get namespace at top of stack
     Chuck_Namespace * nspc_top();
@@ -688,6 +760,14 @@ public:
     Chuck_VM * vm() const { return m_carrier ? m_carrier->vm : NULL; }
     Chuck_Compiler * compiler() const { return m_carrier ? m_carrier->compiler : NULL; }
 
+public:
+    // array type cache
+    Chuck_ArrayTypeCache * arrayTypeCache() { return &array_types; }
+    // retrieve array type based on parameters | 1.5.4.0 (ge, nick, andrew) added
+    Chuck_Type * get_array_type( Chuck_Type * array_parent,
+                                 t_CKUINT depth, Chuck_Type * base_type /*,
+                                 Chuck_Namespace * owner_nspc */ );
+
 protected:
     Chuck_Carrier * m_carrier;
 
@@ -698,6 +778,9 @@ protected:
     Chuck_Context global_context;
     // user-global namespace
     Chuck_Namespace * user_nspc;
+    // cache of various array types, which are created as needed by the type system
+    // e.g., int[] or float[][] or UGen[]; this cache reuses
+    Chuck_ArrayTypeCache array_types;
 
 public:
     // namespace stack
@@ -910,7 +993,7 @@ struct Chuck_Type : public Chuck_Object
     // size (in bytes)
     t_CKUINT size;
     // owner of the type
-    Chuck_Namespace * owner;
+    // Chuck_Namespace * owner;
     // array type
     union { Chuck_Type * array_type; Chuck_Type * actual_type; };
     // array size (equals 0 means not array, else dimension of array)
@@ -918,11 +1001,13 @@ struct Chuck_Type : public Chuck_Object
     // object size (size in memory)
     t_CKUINT obj_size;
     // type info
-    Chuck_Namespace * info;
+    Chuck_Namespace * nspc;
     // func info
     Chuck_Func * func;
     // ugen
     Chuck_UGen_Info * ugen_info;
+    // is public class | 1.5.4.0 (ge) added
+    t_CKBOOL is_public;
     // copy
     t_CKBOOL is_copy;
     // defined
@@ -1346,13 +1431,6 @@ t_CKBOOL type_engine_has_implicit_def_ctor( Chuck_Type * type );
 t_CKUINT type_engine_next_offset( t_CKUINT current_offset, Chuck_Type * type );
 // array verify
 t_CKBOOL verify_array( a_Array_Sub array );
-// make array type
-Chuck_Type * new_array_type( Chuck_Env * env, Chuck_Type * array_parent,
-                             t_CKUINT depth, Chuck_Type * base_type,
-                             Chuck_Namespace * owner_nspc );
-// make type | 1.4.1.1 (nshaheed) added
-Chuck_Type * new_array_element_type( Chuck_Env * env, Chuck_Type * base_type,
-                                     t_CKUINT depth, Chuck_Namespace * owner_nspc);
 
 
 //-----------------------------------------------------------------------------
